from pathlib import Path
from typing import List, Dict, Any

import click
import yaml

from agileupstate.terminal import print_cross_message


class State:

    def __init__(self, client, state_file='siab-state.yml'):
        self.client = client
        self.state_file = state_file

    def file(self):
        return self.state_file

    def data(self) -> dict[str, Any]:
        return {'client-id': self.client.id,
                'client-cloud': self.client.cloud,
                'client-location': self.client.location,
                'client-context': self.client.context,
                }

    def write(self) -> None:
        file = Path(self.state_file)
        with open(file, 'w') as f:
            yaml.dump(self.data(), f, sort_keys=False, default_flow_style=False)

    def read(self) -> dict:
        file = Path(self.state_file)
        if file.is_file():
            click.echo(f'- Reading {file}')
            with open(file, 'r') as f:
                return yaml.safe_load(f)
        else:
            print_cross_message(f'Could not read from state file {file}!', leave=True)



import os
from sys import platform

import click
import pkg_resources

from agileupstate.terminal import print_check_message, print_cross_message


def get_version_string() -> str:
    return f"AgileUp State Version: {pkg_resources.get_distribution('agileupstate').version}"


def supported_os() -> bool:
    if platform == 'linux' or platform == 'linux2':
        return True
    elif platform == 'darwin':
        return True
    elif platform == 'win32':
        return False


class Client:

    def __init__(self):
        self.version = get_version_string()
        try:
            self.id = os.environ['SIAB_ID']
        except KeyError:
            print_cross_message(f'SIAB_ID must be set!', leave=True)
        try:
            self.cloud = os.environ['SIAB_CLOUD']
        except KeyError:
            print_cross_message(f'SIAB_CLOUD must be set!', leave=True)
        try:
            self.location = os.environ['SIAB_LOCATION']
        except KeyError:
            print_cross_message(f'SIAB_LOCATION must be set!', leave=True)
        try:
            self.context = os.environ['SIAB_CONTEXT']
        except KeyError:
            print_cross_message(f'SIAB_CONTEXT must be set!', leave=True)

        if supported_os():
            print_check_message(f'Client machine supported {platform}')
        else:
            print_cross_message(f'Client machine not supported {platform}!', leave=True)

        self.display()

    def display(self) -> str:
        message = f'- Running ({self.id} {self.cloud} {self.location} {self.context}) {self.version}'
        click.secho(message, fg='blue')
        return message



import os

import click
import hvac
from hvac.exceptions import InvalidPath

from agileupstate.client import Client
from agileupstate.state import State
from agileupstate.terminal import print_cross_message, print_check_message


def address():
    return os.environ['VAULT_ADDR']


def is_ready() -> bool:
    client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    if client.sys.is_sealed():
        return print_cross_message(f'Vault is sealed: {address()}', leave=True)
    else:
        if client.is_authenticated():
            print_check_message(f'Vault client authenticated to: {address()}')
            try:
                client.secrets.kv.v2.create_or_update_secret(
                    path='siab/smoke-test',
                    secret=dict(test='this is a vault engine smoke test'),
                )
                list_response = client.secrets.kv.v2.list_secrets(path='siab')
                print('The following paths are available under "siab" prefix: {keys}'.format(
                    keys=','.join(list_response['data']['keys']),
                ))
                return print_check_message(f'Vault client secrets backend validated: {address()}')
            except InvalidPath:
                return print_cross_message(f'Could not find KV v2 siab path in: {address()}', leave=True)

        else:
            return print_cross_message(f'Vault client failed to authenticate to: {address()}', leave=True)


def create_state() -> None:
    client = Client()
    state = State(client)
    state.write()

    client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    state_path = 'siab/' + state.client.id + '-' + state.client.cloud + '-' + state.client.location + '-' + state.client.context
    client.secrets.kv.v2.create_or_update_secret(
        path=state_path,
        secret=state.data()
    )
    click.secho(f'- Created state data in vault {state_path} and file {state.file()}', fg='blue')




import click
import pkg_resources

from agileupstate.client import Client, get_version_string
from agileupstate.state import State
from agileupstate.terminal import print_check_message, print_cross_message
from agileupstate.vault import address, is_ready, create_state

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(help='CLI to manage pipeline states.')
def cli() -> int:
    pass


@cli.command(help='Display the current version.')
def version() -> None:
    click.echo(get_version_string())


@cli.command(help='Check client configuration.')
def check() -> None:
    click.secho('Checking client configuration.', fg='green')
    client = Client()
    if is_ready():
        print_check_message('Vault backend is available')
    else:
        print_cross_message(f'Vault backend failed from: {address()}', leave=True)


@cli.command(help='Create client state and vault path.')
def create() -> None:
    click.secho('- Create client state', fg='green')
    create_state()


@cli.command(help='Update client state.')
def update() -> None:
    click.secho('- Update client state', fg='green')


@cli.command(help='Save client state.')
def save() -> None:
    click.secho('- Save client state', fg='green')


@cli.command(help='Load client state.')
def fetch() -> None:
    click.secho('- Load client state', fg='green')


if __name__ == '__main__':
    exit(cli())



class AgileUpException(Exception):
    """An exception that AgileUp can handle and show to the user."""

    def __init__(self, message: str) -> None:
        super().__init__(message)
        self.message = message

    def format_message(self) -> str:
        return self.message

    def __str__(self) -> str:
        return self.message
